#!/usr/bin/env python
import pygtk
import gtk

import hashlib
import os
import shutil
import sys
import threading
import time

class QuickTeX:
	def __init__(self,template_path):
		self.template_path = template_path
		
	def tex(self,expression,callback=None,user_data=None):
		temp = TeXThread(self.template_path)
		temp.tex(expression,callback,user_data)

class TeXThread(threading.Thread):
	def __init__(self,template_path):
		threading.Thread.__init__(self)
		self.template_path = template_path
		self.dir = os.path.expanduser("~/.quicktex")
		self.cache = self.dir + "/cache"
		if not os.path.exists(self.dir):
			os.mkdir(self.dir)
		if not os.path.exists(self.cache):
			os.mkdir(self.cache)
		self.callback = None
		self.expression = ""
		self.user_data = None
			
	def tex(self,expression,callback=None,user_data=None):
		self.expression = expression
		self.callback = callback
		self.user_data = user_data
		if self.callback == None:
			self.callback = self._dummy
		self.start()
	
	def _dummy(self,val,filename="",user_data=None):
		print val,filename,user_data

	def run(self):
		"""takes a latex expression and converts it to an image.
		callback should be something like
			def callback(percentage,filename)
		returns filename of the created image"""
		hash = hashlib.md5(self.expression).hexdigest()
		fn = self.cache + "/" + hash + ".png"
		self.callback(10)
		if os.path.exists(fn):
			self.callback(100,fn,self.user_data)
		else:
			#0 create temp tex
			f = open(self.template_path)
			template = f.read()
			f.close()
			os.chdir(self.dir)
			template = template.replace("%replace%",self.expression)
			f = open(self.dir + "/%s.tex" % hash,"w")
			f.write(template)
			f.close()
			#1 parse tex
			args = ["latex","-interaction=batchmode","%s.tex" % hash]
			exit_code = os.spawnvp(os.P_WAIT,"latex",args)
			if exit_code != 0:
				self.callback(-1)
			else:
				self.callback(20)
			#2 convert to ps
			if exit_code == 0:
				args = ["dvips","%s.dvi" % hash,"-o","%s.ps" % hash]
				exit_code = os.spawnvp(os.P_WAIT,"dvips",args)
				if exit_code != 0:
					self.callback(-1)
				else:
					self.callback(40)
			#3 convert to bmp
			if exit_code == 0:
				args = ["gs","-r300x300","-dTextAlphaBits=4","-dGraphicsAlphaBits=4",
						"-sDEVICE=bmp16m","-sOutputFile=%s.bmp" % hash,"-dBATCH",
						"-dNOPAUSE","%s.ps" % hash]
				exit_code = os.spawnvp(os.P_WAIT,"gs",args)
				if exit_code != 0:
					self.callback(-1)
				else:
					self.callback(60)
			#4 trim
			if exit_code == 0:
				args = ["mogrify","-trim","%s.bmp" % hash]
				exit_code = os.spawnvp(os.P_WAIT,"mogrify",args)
				if exit_code != 0:
					self.callback(-1)
				else:
					self.callback(80)
			#5 resize and save
			if exit_code == 0:
				args = ["convert","-bordercolor","white","-border","20x20",
						"-antialias","-quality","100","-geometry","35%x35%",
						"%s.bmp" % hash,fn]
				exit_code = os.spawnvp(os.P_WAIT,"convert",args)
				if exit_code != 0:
					self.callback(-1)
				else:
					self.callback(100,fn,self.user_data)
			try:
				os.unlink("%s.tex" % hash)
				os.unlink("%s.aux" % hash)
				os.unlink("%s.log" % hash)
				os.unlink("%s.dvi" % hash)
				os.unlink("%s.ps" % hash)
				os.unlink("%s.bmp" % hash)
			except:
				pass
		
		
class SFQuickTeX:
	def __init__(self):
		#change to rigth directory
		wd = os.getcwd()
		if sys.argv[0].find("/") != -1:
			wd = sys.argv[0][:sys.argv[0].rfind("/")]
			os.chdir(wd)
		self.tex = QuickTeX(wd + "/template")
		self.image_file = ""
		self._init_gui()
		
	def _init_gui_formula(self):
		box = gtk.VBox()
		box_env = gtk.HBox()
		box_buttons = gtk.HButtonBox()
		self.label_environment = gtk.Label("Environment:")
		self.select_environment = gtk.ComboBox()
		self.button_process = gtk.Button(label="Create image")
		self.button_save = gtk.Button(label="Save image as...")
		self.text_buffer = gtk.TextBuffer()
		self.text_view = gtk.TextView(self.text_buffer)
		self.image_output = gtk.Image()
		self.progress = gtk.ProgressBar()
		
		self.image_output.set_from_stock(gtk.STOCK_MISSING_IMAGE,gtk.ICON_SIZE_DIALOG)
		self._create_env_list()
		
		box_env.pack_start(self.label_environment,False,True,2)
		box_env.pack_start(self.select_environment,True,True,2)
		
		box_buttons.add(self.button_process)
		box_buttons.add(self.button_save)
		
		box.pack_start(box_env,False,True,2)
		box.pack_start(gtk.HSeparator(),False,True,2)
		box.pack_start(self.text_view,True,True,2)
		box.pack_start(gtk.HSeparator(),False,True,2)
		box.pack_start(box_buttons,False,True,2)
		box.pack_start(gtk.HSeparator(),False,True,2)
		box.pack_start(self.image_output,False,True,2)
		box.pack_start(self.progress,False,True,2)
		return box
		
	def _init_gui_html(self):
		box = gtk.VBox()
		box_f = gtk.HBox()
		box_buttons = gtk.HButtonBox()
		
		label_info = gtk.Label("This function replaces all LaTeX expressions between <latex>...</latex> in a html file by png images. The LaTeX expressions are processed in an equation* environment.")
		label_info.set_line_wrap(True)
		self.fc = gtk.FileChooserButton("Select HTML file")
		self.fc.set_current_folder(os.path.expanduser("~"))
		label_file = gtk.Label("HTML file:")
		box_buttons = gtk.HButtonBox()
		self.button_process_html = gtk.Button("Process HTML file")
		box_buttons.add(self.button_process_html)
		
		box_f.pack_start(label_file,False,True,2)
		box_f.pack_start(self.fc,True,True,2)
		
		box.pack_start(label_info,False,True,2)
		box.pack_start(box_f,False,True,2)
		box.pack_start(gtk.HSeparator(),False,True,2)
		box.pack_start(box_buttons,False,True,2)
		return box
		
	def _init_gui(self):
		self.window = gtk.Window()
		
		self.window.set_title("SFQuickTeX")
		self.window.set_default_size(400,350)
		self.window.set_position(gtk.WIN_POS_MOUSE)
		self.window.set_border_width(2)
		
		nb = gtk.Notebook()

		nb.append_page(self._init_gui_formula(),gtk.Label("Formulae"))
		nb.append_page(self._init_gui_html(),gtk.Label("HTML"))
		self.window.add(nb)
		self.window.show_all()
		
		self._connect_callbacks()
		gtk.gdk.threads_init()
		gtk.main()
		
	def _connect_callbacks(self):
		self.window.connect("destroy",self._callback_window_closed)
		self.button_process.connect("clicked",self._callback_process)
		self.button_save.connect("clicked",self._callback_save_as)
		self.button_process_html.connect("clicked",self._callback_process_html)
		
	def _create_env_list(self):
		envs = ["None","equation*","eqnarray*"]
		liststore = gtk.ListStore(str)
		renderer = gtk.CellRendererText()
		self.select_environment.pack_start(renderer,True)
		self.select_environment.add_attribute(renderer,'text',0)
		self.select_environment.set_model(liststore)
		for x in envs:
			liststore.set(liststore.append(),0,x)
		self.select_environment.set_active(1)
		
	def _callback_process_html(self,widget):
		self.html_tex_count = 0
		self.html_tex_results = []
		self.html_path = os.path.expanduser(self.fc.get_filename())
		fn = self.html_path
		tex_list = []
		if os.path.exists(fn):
			handler = open(fn,"r")
			markup = handler.read()
			temp = markup
			pos = 0
			pos_end = 0
			offset = 0
			while pos != -1:
				offset = (pos_end + len("</latex>"))
				temp = temp[offset:]
				pos = temp.find("<latex>")
				pos_end = temp.find("</latex>")
				tex = temp[pos:pos_end][len("<latex>"):]
				tex_list.append((pos + offset,pos_end + len("</latex>") + offset,tex))
			handler.close()
			dir = os.path.dirname(self.html_path) + "/latex"
			if not os.path.exists(dir):
				os.mkdir(dir)
			n = 0
			for (start,end,tex) in tex_list:
				if tex != "":
					n = n + 1

			self.html_tex_count = n
			for (start,end,tex) in tex_list:
				if tex != "":
					img_tex = tex
					tex = "\\" + "begin{equation*}\n" + tex.strip() + "\\" + "end{equation*}\n"
					self.tex.tex(tex,self._callback_status_change_html,(start,end,img_tex))

			
	def _callback_status_change_html(self,value,fn="",data=None):
		if fn != "" and data != None:
			self.html_tex_results.append((data[0],data[1],data[2],fn))
			if len(self.html_tex_results) == self.html_tex_count:
				handler = open(self.html_path,"r")
				markup = handler.read()
				handler.close()
				for (start,end,tex,fn) in self.html_tex_results:
					markup = markup.replace("<latex>"+tex+"</latex>",'<img src="latex/' + os.path.basename(fn) + '" alt="' + tex.strip() + '" />')
					shutil.copyfile(fn,os.path.dirname(self.html_path) + "/latex/" + os.path.basename(fn))

				shutil.move(self.html_path,self.html_path + ".bak")
				handler = open(self.html_path,"w")
				handler.write(markup)
				handler.close()
		
	def _callback_window_closed(self,widget=None):
		gtk.main_quit()
		
	def _callback_process(self,widget=None):
		self.progress.set_fraction(0.0)
		text = self.text_buffer.get_text(self.text_buffer.get_start_iter(),self.text_buffer.get_end_iter())
		env = self.select_environment.get_active_text() 
		if env != "None":
			text = "\\" + "begin{%s}\n" % env + text + "\n" + "\\" + "end{%s}" % env
		self.tex.tex(text,self._callback_status_change)
		
	def _callback_status_change(self,value,fn="",user_data=[]):
		if value != -1:
			self.progress.set_fraction(float(value)/100.0)
			self.progress.set_text("%s %%" % value)
		if value == 100 and os.path.exists(fn):
			self.image_output.set_from_file(fn)
			self.image_file = fn
			self.progress.set_text("Done.")
		elif value == -1:
			self.tex.tex("An error occured!\n\n Please check your input.",self._callback_status_change)
			
	def _callback_save_as(self,widget=None):
		f = gtk.FileFilter()
		f.set_name("Image files")
		f.add_pixbuf_formats()
		fc = gtk.FileChooserDialog(action=gtk.FILE_CHOOSER_ACTION_SAVE,buttons=((gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_OK, gtk.RESPONSE_ACCEPT)))
		fc.add_filter(f)
		fc.set_current_name(os.path.basename(self.image_file))
		id = fc.run()
		fn = fc.get_filename()
		fc.destroy()
		if id == -3:
			if fn != None:
				"""if not fn.endswith(".png"):
					fn = fn + ".png"
				shutil.copyfile(self.image_file,fn)"""
				args = ["convert",self.image_file,fn]
				exit_code = os.spawnvp(os.P_WAIT,"convert",args)
			
if __name__ == "__main__":
	app = SFQuickTeX()

